#!/usr/bin/python

import vot

import os
import sys
import argparse

import cv2
import torch
import numpy as np

from pysot.core.config import cfg
from pysot.models.model_builder import ModelBuilder
from pysot.tracker.tracker_builder import build_tracker
from pysot.utils.bbox import get_axis_aligned_bbox

import time
np.random.seed(int(time.time()))

torch.set_num_threads(1)

def search_path(pathname_suffix):
    cands = [os.path.join(d,pathname_suffix) for d in sys.path]
    try:
        return filter(os.path.exists, cands)[0]
    except IndexError:
        return None
        
parser = argparse.ArgumentParser(description='tracking demo')
parser.add_argument('--config', type=str, default=search_path('experiments/siamrpn_r50_l234_dwxcorr_lt/config.yaml'), help='config file')
parser.add_argument('--snapshot', type=str, default=search_path('experiments/siamrpn_r50_l234_dwxcorr_lt/model.pth'), help='model name')
args = parser.parse_args()   


# load config
cfg.merge_from_file(args.config)
cfg.CUDA = torch.cuda.is_available()
device = torch.device('cuda' if cfg.CUDA else 'cpu')

# create model
model = ModelBuilder()

# load model
model.load_state_dict(torch.load(args.snapshot,
    map_location=lambda storage, loc: storage.cpu()))
model.eval().to(device)

# build tracker
tracker = build_tracker(model)

# connect to VOT
handle = vot.VOT("rectangle")
selection = handle.region()

imagefile = handle.frame()
if not imagefile:
    sys.exit(0)


im = cv2.imread(imagefile)
if isinstance(selection, vot.Polygon):
    points = [[p.x, p.y] for p in selection]
    points = np.array(points).flatten()
    cx, cy, w, h = get_axis_aligned_bbox(points)
    x,y,w,h = [cx-(w-1)/2, cy-(h-1)/2, w, h]
    selection = vot.Rectangle(x, y, w, h)

tracker.init(im, [selection.x, selection.y, selection.width, selection.height])

while True:
    imagefile = handle.frame()
    if not imagefile:
        break
    im = cv2.imread(imagefile)
    
    try:
        # track
        outputs = tracker.track(im)
        bbox = list(map(float, outputs['bbox'])) # x,y,w,h
        
        location = np.array([[bbox[0],           bbox[1]          ],
                             [bbox[0] + bbox[2], bbox[1]          ],
                             [bbox[0] + bbox[2], bbox[1] + bbox[3]],
                             [bbox[0],           bbox[1] + bbox[3]]])
        
        region = vot.Polygon([vot.Point(location[x,0], location[x,1]) for x in range(location.shape[0])])
        confidence = outputs['best_score']
        handle.report(region, confidence)
    except:
        handle.report(selection, 0)
