from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os
import sys
from os.path import join, isdir, isfile
import argparse

import cv2
import torch
import numpy as np
from glob import glob

from pysot.core.config import cfg
from pysot.models.model_builder import ModelBuilder
from pysot.tracker.tracker_builder import build_tracker

torch.set_num_threads(1)

parser = argparse.ArgumentParser(description='tracking demo')
parser.add_argument('--config', type=str, default='experiments/siamrpn_r50_l234_dwxcorr_lt/config.yaml', help='config file')
parser.add_argument('--snapshot', type=str, default='experiments/siamrpn_r50_l234_dwxcorr_lt/model.pth', help='model name')
parser.add_argument('--base_path', default='/home/baochen/my_space/vot2019/toolkit-7.0.2/siammask_la_workspace/sequences/rollerman', help='datasets')
#parser.add_argument('--base_path', default='/home/baochen/bk_disk/vot2019/toolkit-7.0.2/siammask_la_workspace/sequences/car1', help='datasets')
parser.add_argument('--other_result_path', default='/home/baochen//my_space/vot2019/toolkit-7.0.2/siamrpn_lt_s_workspace/results/SiamRPN_LT_S_3/longterm/', help='datasets')
#parser.add_argument('--other_result_path', default='/home/baochen/bk_disk/vot2019/toolkit-7.0.2/siamrpnslt_workspace/results/SiamRPNsLT_1/longterm/', help='datasets')
args = parser.parse_args()


def main():
    # load config
    cfg.merge_from_file(args.config)
    cfg.CUDA = torch.cuda.is_available()
    device = torch.device('cuda' if cfg.CUDA else 'cpu')

    # create model
    model = ModelBuilder()

    # load model
    model.load_state_dict(torch.load(args.snapshot,
        map_location=lambda storage, loc: storage.cpu()))
    model.eval().to(device)

    # build tracker
    tracker = build_tracker(model)


    # Parse Image file
    img_files = sorted(glob(join(args.base_path+'/color', '*.*g')))
    #img_files = img_files[1700:]
    
    # Select ROI
    cv2.namedWindow("Tracking")
    cv2.moveWindow("Tracking", 500,100);
    try:
        groundtruth=[]
        if os.path.isfile(args.base_path+'/groundtruth.txt'):
            for line in open(args.base_path+'/groundtruth.txt', 'r'):
                line = line.replace(',', ' ')
                line = line.replace(';', ' ')
                line = line.replace('\t', ' ')
                l = line.split()
                l = [float(x) for x in l]
                try:
                    x, y, w, h  = int(l[1]), int(l[2]), int(l[3]), int(l[4])
                except (IndexError):
                    try:
                        x, y, w, h  = int(l[0]), int(l[1]), int(l[2]), int(l[3])
                    except (ValueError):
                        x, y, w, h = 0, 0, 0, 0
                except (ValueError):
                    x, y, w, h = 0, 0, 0, 0
                groundtruth.append([x, y, w, h])
        else:
            init_rect = cv2.selectROI('Tracking', cv2.imread(img_files[0]), False, False)
            x, y, w, h = init_rect
            groundtruth.append([x, y, w, h])
            
        DaSiam_LT = []
        folder_name = args.base_path.split('/')[-1]
        DaSiam_LT_path = args.other_result_path+folder_name+'/'+folder_name
        if os.path.isfile(DaSiam_LT_path+'_001.txt'):
            for line in open(DaSiam_LT_path+'_001.txt', 'r'):
                line = line.replace(',', ' ')
                line = line.replace(';', ' ')
                line = line.replace('\t', ' ')
                l = line.split()
                l = [float(x) for x in l]
                try:
                    x, y, xmax, ymax  = int(l[0]), int(l[1]), int(l[4]), int(l[5])
                    w = xmax-x
                    h = ymax-y
                except (IndexError):
                    try:
                        x, y, w, h  = int(l[0]), int(l[1]), int(l[2]), int(l[3])
                    except:
                        x, y, w, h = 0, 0, 0, 0
                except:
                    x, y, w, h = 0, 0, 0, 0
                DaSiam_LT.append([x, y, w, h])
    except:
        print('box error')
        exit()
    x, y, w, h = groundtruth[0]
    print(x,y,w,h)


    toc = 0
    for f, imf in enumerate(img_files):
        im = cv2.imread(imf)
        tic = cv2.getTickCount()
        # try:
        if f == 0:  # init
            tracker.init(im, groundtruth[0])  # init tracker
            toc += cv2.getTickCount() - tic
            cv2.rectangle(im, (x, y), (x+w, y+h),  (255, 0, 0), 2)
        elif f > 0:  # tracking
            outputs = tracker.track(im)
            toc += cv2.getTickCount() - tic
            bbox = list(map(int, outputs['bbox'])) # x,y,w,h
            cv2.rectangle(im, (bbox[0],bbox[1]),(bbox[0]+bbox[2], bbox[1]+bbox[3]), (0,255,0),3)
            
            if len(groundtruth) > 1:
                x,y,w,h=groundtruth[f]
                cv2.rectangle(im, (x, y), (x+w, y+h),  (255, 0, 0), 2)
            
            if len(DaSiam_LT) > 1:
                x,y,w,h=DaSiam_LT[f]
                cv2.rectangle(im, (x, y), (x+w, y+h),  (255, 0, 255), 2)
        
            y0, dy = 30, 30
            for i, line in enumerate( outputs['prt_str'].split('\n')):
                y = y0 + i*dy
                cv2.putText(im, line, (30, y ), cv2.FONT_HERSHEY_SIMPLEX, 0.7,(0,255,255),2)
            
            im[im.shape[0]//5*4:,:im.shape[0]//5,:] = cv2.resize(outputs['im_patch'], (im.shape[0]//5,im.shape[0]//5))
            cv2.circle(im, (bbox[0]+bbox[2]//2, bbox[1]+bbox[3]//2), 1, (0, 255, 0), 2)
        cv2.imwrite(args.base_path+'/SiamMask_result/'+str(f).zfill(8)+'.jpg',im)
        cv2.imshow('Tracking', im)
        key = cv2.waitKey(1)
        
        #while key == -1:
        #   key = cv2.waitKey(1)
        if key != -1 and key != 13:
            break
        # except KeyboardInterrupt:
            # break
        # except:
            # pass
        
    toc /= cv2.getTickFrequency()
    fps = f / toc
    print('Time: {:02.1f}s Speed: {:3.1f}fps'.format(toc, fps))


if __name__ == '__main__':
    main()
